@@ -4,7 +4,112 @@ import socket
 import typing
 
 
-class Connector:
+class ThrottleGuard:
+    """Prevent code from executing repeatedly.
+
+    :param threshold: guarding threshold
+
+    This abstraction allows code to execute the first "threshold"
+    times and then only once per "threshold" times afterwards.  Use
+    it to ensure that log statements are continuously written during
+    persistent error conditions.  The goal is to provide regular
+    feedback while limiting the amount of log spam.
+
+    The following snippet will log the first 100 failures and then
+    once every 100 failures thereafter:
+
+    .. code-block:: python
+
+       executions = 0
+       guard = ThrottleGuard(100)
+       for _ in range(1000):
+           if guard.allow_execution():
+               executions += 1
+               logging.info('called %s times instead of %s times',
+                            executions, guard.counter)
+
+    """
+    def __init__(self, threshold: int):
+        self.counter = 0
+        self.threshold = threshold
+
+    def allow_execution(self) -> bool:
+        """Should this execution be allowed?"""
+        self.counter += 1
+        allow = (self.counter < self.threshold
+                 or (self.counter % self.threshold) == 0)
+        return allow
+
+    def reset(self) -> None:
+        """Reset counter after error has resolved."""
+        self.counter = 0
+
+
+class AbstractConnector:
+    """StatsD connector that does not send metrics or connect.
+
+    Use this connector when you want to maintain the application
+    interface without doing any real work.
+
+    """
+    async def start(self) -> None:
+        pass
+
+    async def stop(self) -> None:
+        pass
+
+    def inject_metric(self, path: str, value: str, type_code: str) -> None:
+        pass
+
+    def incr(self, path: str, value: int = 1) -> None:
+        """Increment a counter metric.
+
+        :param path: counter to increment
+        :param value: amount to increment the counter by
+
+        """
+        self.inject_metric(f'counters.{path}', str(value), 'c')
+
+    def decr(self, path: str, value: int = 1) -> None:
+        """Decrement a counter metric.
+
+        :param path: counter to decrement
+        :param value: amount to decrement the counter by
+
+        This is equivalent to ``self.incr(path, -value)``.
+
+        """
+        self.inject_metric(f'counters.{path}', str(-value), 'c')
+
+    def gauge(self, path: str, value: int, delta: bool = False) -> None:
+        """Manipulate a gauge metric.
+
+        :param path: gauge to adjust
+        :param value: value to send
+        :param delta: is this an adjustment of the gauge?
+
+        If the `delta` parameter is ``False`` (or omitted), then
+        `value` is the new value to set the gauge to.  Otherwise,
+        `value` is an adjustment for the current gauge.
+
+        """
+        if delta:
+            payload = f'{value:+d}'
+        else:
+            payload = str(value)
+        self.inject_metric(f'gauges.{path}', payload, 'g')
+
+    def timing(self, path: str, seconds: float) -> None:
+        """Send a timer metric.
+
+        :param path: timer to append a value to
+        :param seconds: number of **seconds** to record
+
+        """
+        self.inject_metric(f'timers.{path}', str(seconds * 1000.0), 'ms')
+
+
+class Connector(AbstractConnector):
     """Sends metrics to a statsd server.
 
     :param host: statsd server to send metrics to
@@ -69,9 +174,11 @@ class Connector:
                  *,
                  prefix: str = '',
                  **kwargs: typing.Any) -> None:
+        super().__init__()
         self.logger = logging.getLogger(__package__).getChild('Connector')
         self.prefix = f'{prefix}.' if prefix else prefix
         self.processor = Processor(host=host, port=port, **kwargs)
+        self._enqueue_log_guard = ThrottleGuard(100)
         self._processor_task: typing.Optional[asyncio.Task[None]] = None
 
     async def start(self) -> None:
@@ -95,53 +202,6 @@ class Connector:
         """
         await self.processor.stop()
 
-    def incr(self, path: str, value: int = 1) -> None:
-        """Increment a counter metric.
-
-        :param path: counter to increment
-        :param value: amount to increment the counter by
-
-        """
-        self.inject_metric(f'counters.{path}', str(value), 'c')
-
-    def decr(self, path: str, value: int = 1) -> None:
-        """Decrement a counter metric.
-
-        :param path: counter to decrement
-        :param value: amount to decrement the counter by
-
-        This is equivalent to ``self.incr(path, -value)``.
-
-        """
-        self.inject_metric(f'counters.{path}', str(-value), 'c')
-
-    def gauge(self, path: str, value: int, delta: bool = False) -> None:
-        """Manipulate a gauge metric.
-
-        :param path: gauge to adjust
-        :param value: value to send
-        :param delta: is this an adjustment of the gauge?
-
-        If the `delta` parameter is ``False`` (or omitted), then
-        `value` is the new value to set the gauge to.  Otherwise,
-        `value` is an adjustment for the current gauge.
-
-        """
-        if delta:
-            payload = f'{value:+d}'
-        else:
-            payload = str(value)
-        self.inject_metric(f'gauges.{path}', payload, 'g')
-
-    def timing(self, path: str, seconds: float) -> None:
-        """Send a timer metric.
-
-        :param path: timer to append a value to
-        :param seconds: number of **seconds** to record
-
-        """
-        self.inject_metric(f'timers.{path}', str(seconds * 1000.0), 'ms')
-
     def inject_metric(self, path: str, value: str, type_code: str) -> None:
         """Send a metric to the statsd server.
 
@@ -156,8 +216,10 @@ class Connector:
         payload = f'{self.prefix}{path}:{value}|{type_code}'
         try:
             self.processor.enqueue(payload.encode('utf-8'))
+            self._enqueue_log_guard.reset()
         except asyncio.QueueFull:
-            self.logger.warning('statsd queue is full, discarding metric')
+            if self._enqueue_log_guard.allow_execution():
+                self.logger.warning('statsd queue is full, discarding metric')
 
 
 class StatsdProtocol(asyncio.BaseProtocol):
@@ -371,6 +433,7 @@ class Processor:
         self.host = host
         self.port = port
         self._ip_protocol = ip_protocol
+        self._connect_log_guard = ThrottleGuard(100)
         self._reconnect_sleep = reconnect_sleep
         self._wait_timeout = wait_timeout
 
@@ -461,14 +524,21 @@ class Processor:
                 buffered_data = b''
                 if self.protocol is not None:
                     buffered_data = self.protocol.buffered_data
+
                 t, p = await self._create_transport()  # type: ignore[misc]
                 transport, self.protocol = t, p
                 self.protocol.buffered_data = buffered_data
-                self.logger.info('connection established to %s',
-                                 transport.get_extra_info('peername'))
+                self.logger.info(
+                    'connection established to %s after %s attempts',
+                    transport.get_extra_info('peername'),
+                    self._connect_log_guard.counter)
+                self._connect_log_guard.reset()
             except IOError as error:
-                self.logger.warning('connection to %s:%s failed: %s',
-                                    self.host, self.port, error)
+                if self._connect_log_guard.allow_execution():
+                    self.logger.warning(
+                        'connection to %s:%s failed: %s (%s attempts)',
+                        self.host, self.port, error,
+                        self._connect_log_guard.counter)
                 await asyncio.sleep(self._reconnect_sleep)
 
     async def _process_metric(self) -> None:
